# File tres/Scripts/stres.cl
# March 10, 2009
# By Doug Mink, SAO Telescope Data Center

# STRES - Process multiple similar TRES sky flat images from raw data files

procedure stres (images)

string	images=""	{prompt="Images to process"}
string	datedir=""	{prompt="Date for images"}
string	instdir=""	{prompt="Directory containing date directory"}
bool	fixpreamp=no	{prompt="Run tpreamp to fix time constant (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
real	gaincorr=0.95	{prompt="Gain correction for amp2 to amp1"}
bool	flatten=no	{prompt="Run TFLAT to Flatten image (yes or no)"}
string	normfile=""	{prompt="Pixel to pixel tmakeflat normalization file"}
string	normdir="tresdata$"	{prompt="Directory for normalization file"}
real	threshold=10.0	{prompt="Cosmic ray threshold in sigma"}
bool	avgspec=yes	{prompt="Compute a mean image (yes or no)"}
bool	sumspec=no	{prompt="Sum these images (yes or no)"}
bool	makeref=yes	{prompt="Make aperture reference(s) and flattening file from sum"}
bool	interact=yes	{prompt="Run tmakeref interactively (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	delimage=no	{prompt="Delete processed image file (yes or no)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=no	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print process progress (y or n)"}

begin

file	imroot, imin, impath, flatfile
file	imout, ims, cdir, tempin, tempout
string	ddir, idir, croot
bool	vb, dt, sr, sv, disp, rc, sum, sumonly, delim, avg, db
real	ct

	vb = verbose
	db = debug
	dt = deltemp
	sv = saveold
	sr = saveraw
	ddir = datedir
	idir = instdir
	ims = images
	delim = delimage
	ct = threshold
	avg = avgspec
	sum = sumspec

	if (substr (ims,1,1) == "@")
	    fileroot (substr (ims,2,100))
	else
	    fileroot (ims)
	trssum.outroot = fileroot.rootname

	if (ims == "") {
	    print ("FTRES: Reduce multiple similar TRES flat field images, removing cosmic rays")
	    print ("Usage: ftres images with both amps to be corrected")
	    return
	    }

# If input image is a list prepend @ if not already there
	smatch (ims, ".list")
	if (smatch.found) {
	    if (substr (ims,1,1) != '@')
		ims = "@" // ims
	    }

	tcheck
	if (tcheck.rawdir)
	    trsdate (tcheck.datedir)

#  Make list file from input file parameter
	trsfiles.instdir = idir
	trsfiles.datedir = ddir
	trsfiles.copyfile = yes
	trsfiles.overwrite = no
	trsfiles (ims)
	if (trsfiles.nfiles == 0) {
	    delete (trsfiles.tempin)
	    if (vb)
		print ("FTRES: No files found: ", ims)
	    return
	    }
	tempin = trsfiles.tempin
	tempout = trsfiles.tempout
	ims = "@" // tempout

# Remove overscan, trim, and merge amplifiers of image
# Then remove pixel to pixel variation from image (including IR fringing)
# Then extract spectra from image and add dispersion function
	tproc.gaincorr = gaincorr
	tproc.fixbias = fixbias
	tproc.fixpix = fixpix
	tproc.fixpreamp = fixpreamp
	tproc.flatten = no
	tproc.cosmic = yes
	tproc.threshold = ct
	if (sum)
	    tproc.sumspec = "yes"
	else
	    tproc.sumspec = "no"
	tproc.avgspec = avg
	tproc.deltemp = dt
	tproc.saveraw = sr
	tproc.saveold = sv
	tproc.verbose = vb
	tcosmic.savebp = sv
	tproc (ims, flatten=no, extract=no, disperse=no)

# Compute average file if requested
	if (avg) {
	    if (trsavg.nfiles == 1)
		print ("FTRES: No average computed from one file ",trsavg.outfile)
	    }

# Compute extraction aperture map and flattening image for average or sum
	if (makeref && (sum || avg)) {
	    if (sum)
		flatfile = trssum.outfile
	    else
		flatfile = trsavg.outfile
	    tmakeref.idbdir = "tresdata$"
	    tmakeref.idbroot = "flat$"
	    tmakeref.interact = interact
	    tmakeref.makeflat = no
	    tmakeref.split = yes
	    tmakeref.verbose = vb
	    tmakeref.debug = db
	    tmakeref (flatfile)
	    }

	if (delim) {
	    imdelete (ims)
	    }

#  Delete temporary list files
	if (dt) {
	    delete (tempin)
	    delete (tempout)
	    }

	if (tcheck.rawdir)
	    tdata (datedir=tcheck.datedir)
end

# Mar 10 2009	New task based on FTRES
