# File tres/Scripts/tcal1.cl
# March 6, 2009
# By Doug Mink

#  TCAL1 -- Make a local id database file for a calibration spectrum

procedure tcal1 (spectrum)

string  spectrum=""       {prompt="Images to check"}

string	refroot="comp"	{prompt="Root name for comparison reference file"}
string	refdir="tresdata$" {prompt="Directory for comparison reference file"}
file	imref=""	{prompt="Reference spectrum image (returned)"}
real	pshift=INDEF	{prompt="Pixel shift from reference file (fit if INDEF)"}
real	shift=INDEF	{prompt="Pixel shift (returned)"}
string	mjdmid=""	{prompt="MJD at midtime of exposure (returned)"}
bool	medshift=no	{prompt="yes: median shift no: mean shift over orders"}
bool	interactive=no	{prompt="Run ecidentify on spectrum (yes or no)"}
bool	reidentify=no	{prompt="Rerun ecreidentify after xcorr (yes or no)"}
bool	redo=no		{prompt="Rerun tcreid even if already run (yes or no)"}
bool	keepshift=no	{prompt="Keep TIDSHIFT shift if yes or cross-correlate"}
bool	disperse=yes	{prompt="Dispersion correct extracted spectra (yes or no)"}
int	ncols=65536	{prompt="Number of columns into which to rebin"}
bool	saveold = yes	{prompt="Save original file to Nodisp/ (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image lists (yes or no)"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print debugging information (yes or no)"}
bool    xdebug=no	{prompt="Print xc debugging information (yes or no)"}

begin

    string imin, calref, history, ecref, refid
    file impix, newfile, rdir, dbref, compref, refthar, dbfile
    bool vb, db, xdb, ecid, sv, dt, rd, rerun, mshift, reid
    int nc

    imin = spectrum
    rdir = refdir
    ecid = interactive
    nc = ncols
    dt = deltemp
    rd = redo
    reid = reidentify
    sv = saveold
    db = debug
    xdb = xdebug
    mshift = medshift
    if (db)
	vb = db
    else
	vb = verbose

# Skip this file if it has already been dispersion-corrected
    imgetp.default = "0"
    imgetp (imin, "TDISP")
    if (imgetp.value != "0") {
	if (vb) {
	    print ("TCAL1: ", imin, " has been dispersion-corrected")
	    print ("TDISP = ", imgetp.value)
	    }
	imgetp.default = "0"
	imgetp (imin, "TIDSHIFT")
	if (vb) {
	    print ("TIDSHIFT = ", imgetp.value)
	    }
	if (shift != INDEF) {
	    shift = real (imgetp.value)
	    if (!rd)
		return
	    rerun = keepshift
	   }
	else
	    rerun = yes
	}
    else if (pshift == INDEF) {
	rerun = no
	shift = pshift
	}
    else
	rerun = yes

# Check for the existence of the reference spectrum locally
    if (imref == "" || !access (imref)) {
	tidref (imin, refroot=refroot, refdir="", verbose=vb, debug=db)
	if (tidref.reffile == "")
	    tidref (imin, refroot=refroot, refdir=refdir, verbose=vb, debug=db)
	imref = tidref.reffile
	cround (tidref.mjdmid, "%12.6f")
	mjdmid = cround.value
	}
    else {
	tfib1 (imref)
	cround (tfib1.mjdmid, "%12.6f")
	mjdmid = cround.value
	}
    fileroot (tidref.reffile)
    dbref = "database/ec" // fileroot.rootname

# If reference spectrum does not exist locally, copy it and id database file
    if (access (imref) && access (dbref)) {
	rdir = ""
	if (db)
	    print ("TCAL1: Reference spectrum is ",imref)
	}
    else {
	if (access (tidref.reffile)) {
	    delete (tidref.reffile)
	    fileroot (tidref.reffile)
	    dbfile = "database/ec" // fileroot.rootname
	    if (access (dbfile))
		delete (dbfile)
	    if (vb)
		print ("TCAL1: Deleting ", tidref.reffile)
	    }
	copy (tidref.refpath, tidref.reffile)
	tmodid (tidref.reffile,tidref.reffile,shift=0.0,scale=1.0,indir=rdir)
	if (db)
	    print ("TCAL1: Reference spectrum is ",rdir,tidref.reffile)
	rdir = ""
	}

# Save name of reference spectrum in the header
    tfib1 (imin)
    compref = "comp" // tfib1.apfib // ".ec.fits"
    if (imref == compref) {
	imgetp.default = compref
	imgetp (imref, "COMPFILE")
	refthar = imgetp.value
	}
    else
	refthar = imref
    hedit (imin, "TIDREF", refthar, add+, update+,verify-,show=db)

# Save the mid-time of the reference spectrum as MJD in the header
    hedit (imin, "TIDMJD", mjdmid, add+, update+,verify-,show=db)

# If no shift is known, find it by cross-correlating against a ThAr standard
    if (!rerun) {

# Load the RVSAO package for pixel space cross-correlation task
	if (!deftask ("pxcsao")) {
	    rvsao (motd=no)
	    }
	pxcsao.spectra = imin
	pxcsao.specnum = "1-48"
	pxcsao.specband = 0
	pxcsao.specdir = ""
	pxcsao.correlate = "pixel"
	pxcsao.templates = imref
	pxcsao.tempnum = "1-48"
	pxcsao.tempband = 1
	pxcsao.tempdir = rdir
	pxcsao.echelle = yes
	pxcsao.st_lambda = INDEF
	pxcsao.end_lambda = INDEF
	pxcsao.obj_plot = no
	pxcsao.xcor_plot = no
	pxcsao.xcor_file = no
	pxcsao.fixbad = no
	pxcsao.badlines = "tharskip.dat"
	pxcsao.s_emchop = "no"
	pxcsao.t_emchop = "no"
	pxcsao.s_abs_reject = 100.
	pxcsao.s_em_reject = 2.
	pxcsao.t_abs_reject = 0.
	pxcsao.t_em_reject = 0.
	pxcsao.bell_window = 0.05
	pxcsao.renormalize = no
	pxcsao.ncols = nc
	pxcsao.interp_mode = "sums"
	pxcsao.zeropad = "yes"
	pxcsao.low_bin = 5
	pxcsao.top_low = 20
	pxcsao.top_nrun = nc
	pxcsao.nrun = nc
	pxcsao.vel_init = "zero"
	pxcsao.czguess = 0.
	pxcsao.nzpass = 0
	pxcsao.tshift = 0.
	pxcsao.svel_corr = "none"
	pxcsao.tvel_corr = "none"
	pxcsao.pkmode = 3
	pxcsao.pkfrac = 0.95
	pxcsao.pksrch = 25
	pxcsao.minvel = -30.
	pxcsao.maxvel = 30.
	pxcsao.report_mode = 18
	pxcsao.logfiles = "STDOUT,xcsao.log"
	pxcsao.save_vel = no
	pxcsao.rvcheck = no
	pxcsao.archive = no
	pxcsao.nsmooth = 0
	pxcsao.cvel = INDEF
	pxcsao.dvel = INDEF
	pxcsao.ablines = ""
	pxcsao.emlines = "thar.dat"
	pxcsao.linedir = "tres$lib/"
	pxcsao.dispmode = 1
	pxcsao.displot = xdb
	pxcsao.device = "stdgraph"
	pxcsao.vel_plot = "correlation"
	pxcsao.curmode = no
	pxcsao.hardcopy = no
	pxcsao.plotter = "stdplot"
	pxcsao.besttemp = ""
	pxcsao.velocity = 0.0
	pxcsao.velerr = 0.0
	pxcsao.r = 0.0
	pxcsao.temp_plot = no
	pxcsao.contsub_plot = no
	pxcsao.apodize_plot = no
	pxcsao.fft_plot = no
	pxcsao.tfft_plot = no
	pxcsao.uxcor_plot = no
	pxcsao.nsum = 1
	pxcsao.debug = xdb

# Cross correlate with reference calibration spectrum
	pxcsao (imin)
	if (mshift)
	    shift = pxcsao.velmed
	else
	    shift = pxcsao.velocity
	}

# Copy reference database/ec file with shift from correlation or header
    tmodid.indir = rdir
    tmodid.outdir = ""
    tmodid.shift = shift
    hedit (imin, "TIDSHIFT", shift, add+, update+,verify-,show=db)
    tmodid.scale = 1.0
    if (ecid)
	tmodid.keepfit = no
    else
	tmodid.keepfit = yes
    tmodid.verbose = vb
    tmodid.debug = db
    tmodid (imref, imin)

# Adjust line fit interactively, if requested
    fileroot (imin)
    calref = fileroot.rootname
    hedit (imin, "REFSPEC1", calref, add+, update+,verify-,show=db)
    if (ecid) {
	ecidentify.database = "database"
	ecidentify.coordlist = "tres$lib/thar.dat"
	ecidentify.units = ""
	ecidentify.match = 1.
	ecidentify.maxfeatures = 100
	ecidentify.zwidth = 10.
	ecidentify.ftype = "emission"
	ecidentify.fwidth = 4.
	ecidentify.cradius = 5.
	ecidentify.threshold = 10.
	ecidentify.minsep = 2.
	ecidentify.function = "chebyshev"
	ecidentify.xorder = 4
	ecidentify.yorder = 4
	ecidentify.niterate = 5
	ecidentify.lowreject = 3.
	ecidentify.highreject = 3.
	ecidentify.autowrite = no
	ecidentify.graphics = "stdgraph"
	ecidentify (imin)
	}
    else if (reid) {
	fileroot (imref)
	refthar = fileroot.rootname
	ecreidentify.refit = yes
	ecreidentify.shift = shift
	ecreidentify.cradius = 5.
	ecreidentify.threshold = 10.
	ecreidentify.database = "database"
	ecreidentify (imin, refthar)
	}
    else {
	ecrefit (imin)
	}

# Set dispersion task parameters
    if (disperse) {
	dispcor.linearize = no
	dispcor.database = "database"
	dispcor.table = ""
	dispcor.w1 = INDEF
	dispcor.w2 = INDEF
	dispcor.dw = INDEF
	dispcor.nw = INDEF
	dispcor.log = no
	dispcor.flux = yes
	dispcor.blank = 0.
	dispcor.samedisp = no
	dispcor.global = no
	dispcor.ignoreaps = no
	dispcor.confirm = no
	dispcor.listonly = no
	dispcor.logfile = ""

	fileroot (calref)
	impix = fileroot.rootname // "p.ec.fits"
	rename (imin, impix)
	dispcor (impix, imin, linearize=no, verbose=vb)
	if (!access (imin)) {
	    print ("TCAL1: DISPCOR failed on ",imin)
	    rename (impix, imin)
	    return
	    }

# Set shift in header to zero if dispersion is refit
	if (reid || ecid) {
	    shift = 0.0
	    hedit (imin, "TIDSHIFT", shift, add+, update+,verify-,show=db)
	    gettime (tfib1.mjd)
	    refid = gettime.datadir // "/" // imin
	    hedit (imin, "TIDREF", refid, add+, update+,verify-,show=db)
	    }

# Add history line including time
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " dispersion fit from " // calref
	hedit (imin, "TDISP", history, add+, verify-, show=db)

# Move original file to Nodisp directory
	if (sv) {
	    if (!access ("Nodisp/")) {
		mkdir ("Nodisp")
		}
	    newfile = "Nodisp/" // imin
	    if (access (newfile)) {
		imdelete (newfile)
		}
	    rename (impix, newfile)
	    if (vb) {
		print ("TCAL1: Original ", imin, " => Nodisp/")
		}
	   }
	else if (dt) {
	    delete (impix)
	    }
	}
    return
end

# Jan 28 2008	New task
# Jan 29 2008	Add saveold to save undispersed spectrum to Nodisp/
# Jan 29 2008	Write TCAL FITS parameter to output file
# Mar 11 2008	Rename task TCAL1
# Mar 11 2008	Do not run task if the file is already dispersion-corrected
# Mar 15 2008	Add shift parameter to return computed shift for re-use
# Mar 15 2008	Add interactive parameter to make running ecidentify optional
# Apr  2 2008	Fix bug by adding deltemp parameter
# May 16 2008	Add input time argument to gettime() 
# May 22 2008	Add TIDREF keyword with path to reference ThAr spectrum
# Jul 16 2008	Return mjdmid from TIDREF
# Jul 16 2008	Add TIDMJD keyword with MJD of reference spectrum
# Jul 17 2008	Return reference image name as imref
# Jul 29 2008	Add error return if DISPCOR fails
# Jul 30 2008	Add ncols parameter to speed processing
# Aug 11 2008	Add redo parameter to rerun calibration with existing shift
# Aug 13 2008	Set all ecidentify parameters if interactive=yes
# Aug 14 2008	Always recompute shift if redo=yes
# Aug 15 2008	Make dispersion correction optional with disperse parameter
# Aug 15 2008	Add keepshift parameter to avoid new cross-correlation
# Aug 15 2008	Use new ECREFIT task to refit lines
# Sep  9 2008	Add xdebug parameter to separate pxcsao debugging
# Sep 12 2008	Add pshift parameter to set a specific pixel shift
# Sep 19 2008	Add medshift parameter to allow option of median not mean
# Oct 20 2008	Set shift limits between -20 and +20 km/sec; use orders 1-48
# Dec 15 2008	Add timediff argument to GETTIME to get current time
# Dec 18 2008	Put date and sequence of ThAr file in header TIDREF
# Dec 18 2008	Return date and sequence of ThAr file as parameter imref
# Dec 30 2008	Add parameter reidentify to run ecreidentify instead of ecidentify
# Dec 31 2008	Set TIDSHIFT in header to zero if reidentify or interactive set

# Feb 19 2009	Add option to specify a specific local reference ThAr
# Mar  2 2009	Fix new comp* file
# Mar  5 2009	Return imref=comp<apfib>.ec.fits if that file is present
# Mar  6 2009	Replace call to apfib1 with call to tfib1
# Mar  6 2009	Check for ecid before reid so interaction works
