# File tres/Scripts/tdisp.cl
# By Doug Mink, SAO Telescope Data Center
# March 9, 2009

# TDISP -- Dispersion correct list of TRES spectra

procedure tdisp (spectra)

string	spectra = ""	{prompt="Input multispec image(s) to correct"}

string	refpath=""	{prompt="Path name for reference comparison file"}
string	refroot="comp"	{prompt="Root name for comparison reference file"}
string	refdir="tresdata$" {prompt="Directory for comparison reference file"}
string	refselect="interp" {prompt="Method (closest, interp, fit, reference)"}

bool saveold = yes	{prompt="Save original file to Nonorm/ (y or n)"}
bool deltemp = no	{prompt="Delete temporary files (y or n)"}
bool verbose = yes	{prompt="Log progress (y or n)"}

struct *dispfiles

begin
	file imin, templist, imout
	string ims, imroot, fib1, fib2
	real shift

# If input image is a list prepend @ if not already there
	ims = spectra
        smatch (ims, ".list")
        if (smatch.found) {
            if (substr (ims,1,1) != '@')
		ims = "@" // ims
            }

	tdisp1.refpath = refpath
	tdisp1.refroot = refroot
	tdisp1.refdir = refdir
	tdisp1.refselect = refselect
	tdisp1.saveold = saveold
	tdisp1.verbose = verbose
	shift = 0.0

#  Make temporary list of input files
        templist = mktemp ("tempcomp") // ".list"
        files (ims, >templist)

        dispfiles = templist
        while (fscan (dispfiles,imin) != EOF) {
	    smatch (imin, ".ec.")
	    if (smatch.found) {
		if (verbose)
		    print ("TDISP: Processing ", imin)
		tdisp1 (imin)
		shift = shift + tdisp1.shift
		}
	    else {
		tfib1 (imin)
		fib1 = tfib1.fib1
		fib2 = tfib1.fib2
		fileroot (imin)
		imroot = fileroot.rootname
		if (fib1 != "COMP") {
		    imout = imroot // "_f1.ec.fits"
		    if (access (imout)) {
			if (verbose)
			    print ("TDISP: Processing fiber 1 ", imout)
			tdisp1 (imout)
			}
		    else if (verbose)
			print ("TDISP: Cannot read fiber 1 file ", imout)
		    }
		else if (verbose)
		    print ("TDISP: Fiber 1 file ", imout, " not object or sky")
		if (fib2 !=  "COMP") {
		    imout = imroot // "_f2.ec.fits"
		    if (access (imout)) {
			if (verbose)
			    print ("TDISP: Processing fiber 2 ", imout)
			tdisp1 (imout)
			}
		    else if (verbose)
			print ("TDISP: Cannot read fiber 2 file ", imout)
		    }
		else if (verbose)
		    print ("TDISP: Fiber 2 file ", imout, " not object or sky")
		}
	    }

# Delete temporary file list if requested
	if (deltemp) {
	    delete (templist)
	    }
	dispfiles = ""

end

# Jul 21 2008	New task
# Jul 28 2008	Change parameter refmode to refselect
# Aug 13 2008	Name files _f1 _f2 _f3 instead of _fib1 _fib2 _fib3
# Aug 13 2008	Use TFIB1 instead of TRSFIB1
# Aug 14 2008	Assign fib1-3 before running TDISP1; it runs tfib1, too
# Aug 14 2008	Assign imroot before running TDISP1; it runs fileroot, too
# Oct 20 2008	Fix bug by using correct parameter fileroot.rootname
# Dec 31 2008	Make @ optional for *.list file input

# Feb  6 2009	Check for fibi !=COMP instead of =SKY or =OBJ
# Mar  9 2009	Fix typo
