# TDISPREF -- Find reference calibration lamp spectrum for a TRES spectrum
# By Doug Mink, SAO Telescope Data Center
# June 6, 2008

procedure tdispref (spectrum)

file	spectrum=""	{prompt="Spectrum for which to find reference"}

file	refroot="comp"	{prompt="Root filename for dispersion reference spectrum"}
file	refdir="tresdata$"	{prompt="Directory for reference spectra"}
file	refpath=""	{prompt="Dispersion reference file (returned)"}
file	reffile=""	{prompt="Dispersion reference file (returned)"}
bool	verbose=yes	{prompt="Log progress (y or n)"}
bool	debug=yes	{prompt="Print debugging information (y or n)"}

begin
	bool vb, db
	string rroot, binning, apfib
	file rdir, spec, dbfile, dbpath
	string fiber

	spec = spectrum
	rdir = refdir
	fixdir (rdir)
	rdir = fixdir.newdir
	if (rdir == "") {
	    rdir = "tresdata$"
	    }
	rroot = refroot
	if (rroot == "") {
	    rroot = "comp"
	    }
	vb = verbose
	db = debug

# Get fiber, binning, and aperture size for this spectrum
	tfib1 (spec, specdir="", verbose=db, debug=db)
	fiber = tfib1.fiber
	binning = tfib1.binning
	apfib = tfib1.apfib

# Get file name for fiber reference image
	reffile = rroot // apfib // ".ec.fits"
	refpath = rdir // reffile
	dbfile = "ec" // rroot // apfib // ".ec"
	dbpath = rdir // "database/" // dbfile
	if (!access (refpath) || !access (dbpath)) {
	    if (db) {
		if (!access (refpath))
		    print ("TDISPREF: ",refpath," does not exist")
		if (!access (dbpath))
		    print ("TDISPREF: ",dbpath," does not exist")
		}
	    reffile = rroot // binning // fiber // ".ec.fits"
	    refpath = rdir // reffile
	    dbfile = "ec" // rroot // apfib // ".ec"
	    dbpath = rdir // "database/" // dbfile
	    if (!access (refpath) || !access (dbpath)) {
		if (db) {
		    if (!access (refpath))
			print ("TDISPREF: ",refpath," does not exist")
		    if (!access (dbpath))
			print ("TDISPREF: ",dbpath," does not exist")
		    }
		reffile = rroot // binning // "1.ec.fits"
		refpath = rdir // reffile
		dbfile = "ec" // rroot // binning // "1.ec.fits"
		dbpath = rdir // "database/" // dbfile
		if (!access (refpath) && !access (dbpath)) {
		    reffile = ""
		    if (vb)
			print ("TDISPREF: ",refpath," and ",dbpath," do not exist")
		    return
		    }
		if (!access (refpath)) {
		    reffile = ""
		    if (vb)
			print ("TDISPREF: ",refpath," does not exist")
		    return
		    }
		if (!access (dbpath)) {
		    reffile = ""
		    if (vb)
			print ("TDISPREF: ",dbpath," does not exist")
		    return
		    }
		}
	    }
	if (vb) {
	    if (db) {
		print ("TDISPREF: Calibration reference file is ", refpath)
		print ("TDISPREF: Calibration database id file is ", dbpath)
		}
	    else
		print (refpath)
	    }

	return
end

# Jan 30 2008	New task
# Feb 13 2008	Set specdir="" in call to trsfib
# Feb 13 2008	Separate checks for reference file and reference database file
# Jun  6 2008	Call TRSFIB1 instead of TRSFIB
# Jun  6 2008	Use tfib1.apfib instead of concatenating fiber, bin, size
