# TFIB - Print apfib keyword for spectra from a list
# August 11, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tfib (spectra)

string	spectra=""	{prompt="Spectra to process"}

file	specdir=""	{prompt="Input directory for data files"}
bool	addfib=yes	{prompt="Add FIBi keywords to spectra (yes or no)"}
string	imagetype=""	{prompt="IRAF image type (returned)"}
string	dateobs=""	{prompt="UT date and time of observation (returned)"}
string	object=""	{prompt="Object name (returned)"}
string	apfib=""	{prompt="Aperture size (s m l) binning (b or null) fiber (1 2 3) (returned)"}
real	mjd=0.0		{prompt="Modified Julian Day of start of observation (returned)"}
real	mjdmid=0.0	{prompt="Modified Julian Day of midtime of observation (returned)"}
string	peri="IN" 	{prompt="Periscope IN (Fibers 1 and 2) or OUT (fiber 3) (returned)"}
string	bin1="1"	{prompt="Binning (1 or 2) (returned)"}
string	fibsize="1.4"	{prompt="Fiber size (1.3 2.3 2.8) (returned)"}
string	osswitch="NO"	{prompt="NO: 1=object 2=sky  YES: 1=sky 2=object (returned)"}
string	exptime=""	{prompt="Exposure time in seconds (returned)"}
string	comptime="0.0"	{prompt="Comparison lamp flash (peri=OUT) time in sec (returned)"}
string	fibkey="NNN"	{prompt="Fiber content (returned)"}
string	fiber=""	{prompt="Fiber(s) present in this image/spectrum (returned)"}
string	fib1="OFF"	{prompt="Fiber 1 (usually object) content (returned)"}
string	fib2="OFF"	{prompt="Fiber 2 (usually sky) content (returned)"}
string	fib3="OFF"	{prompt="Fiber 3 (usually comp) content (returned)"}
string	binning=""	{prompt="TRES binning (b if 2 null if 1) (returned)"}
string	apsize="s"	{prompt="TRES aperture size (s=1.8 m=2.3 l=2.9) (returned)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Log progress (yes or no)"}
bool	debug=no	{prompt="Print debugging information (yes or no)"}

struct *fibfiles
begin

file	imin, templist, sd
string	specs
bool	vb, dt, db, aaf

	specs = spectra
	sd = specdir
	aaf = addfib
	vb = verbose
	dt = deltemp
	db = debug

	if (specs == "") {
	    print ("TFIB: Set header keywords to ease TRES processing")
	    print ("Usage: tfib <input spectra>")
	    return
	    }

#  Make temporary list of input files
	templist = mktemp ("tempfib") // ".list"
	files (specs, >templist)

# Correct list of files for preamp time constant
	fibfiles = templist
        while (fscan (fibfiles, imin) != EOF) {
	    tfib1 (imin,specdir=sd,addfib=aaf,verbose=vb,debug=db)
	    if (!vb && !db)
		print (imin, "	",tfib1.apfib)
	    }

# Set returned parameters
	imagetype = tfib1.imagetype
	dateobs = tfib1.dateobs
	object = tfib1.object
	apfib = tfib1.apfib
	mjd = tfib1.mjd
	mjdmid = tfib1.mjdmid
	peri = tfib1.peri
	bin1 = tfib1.bin1
	fibsize = tfib1.fibsize
	osswitch = tfib1.osswitch
	exptime = tfib1.exptime
	comptime = tfib1.comptime
	fibkey = tfib1.fibkey
	fiber = tfib1.fibkey
	fib1 = tfib1.fib1
	fib2 = tfib1.fib2
	fib3 = tfib1.fib3
	binning = tfib1.binning
	apsize = tfib1.apsize

	if (dt)
	    delete (templist)
	fibfiles = ""
end

# Jun  6 2008	New task to call TRSFIB for multiple files
# Aug  1 2008	Print apfib (apertures, size, binning)
# Aug 11 2008	Change name from trsfib to tfib; call tfib1 instead of tfib1
