# File tres/Scripts/tflatlist.cl
# May 23, 2008
# By Doug Mink

# TFLATLIST - Make files with lists of FLAT images split by aperture size, fiber(s), exposure time, and binning

procedure tflatlist (images)

string  images="*.fits"       {prompt="Images to check"}

string	apfib=""	{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
string	fibcode=""	{prompt="FIBSIZE value to match null=all s=1.4 m=2.3 l=3.2"}
int	binning=0	{prompt="BINNING value to match 0=all 1 2"}
string  apertures="123" {prompt="Apertures to use: null=all 1 2 3 12 123"}
string	exptime=""	{prompt="Exposure time to list (0.0 = all)"}
bool	overwrite=no	{prompt="Overwrite existing list files (yes or no)"}
bool	deltemp=yes	{prompt="Delete temporary files (yes or no)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}
bool	debug=no	{prompt="Print debugging info (yes or no)"}

begin

    string ims, aps, afx, exp, outfile
    bool vb, db, dt, ow

    ims = images
    vb = verbose
    db = debug
    dt = deltemp
    ow = overwrite
    exp = exptime

    afx = apfib
    if (afx != "") {
	trsdecode (afx)
	binning = trsdecode.binning
	fibcode = trsdecode.fibcode
	aps = trsdecode.apertures
	}
    else {
	aps = apertures
	if (aps == "")
	    aps = "123"
	}
    if (db)
	print ("TFLATLIST: ap =",aps," fiber size= ", fibcode," binning= ",binning)


# Largest fiber flat field images
    if (fibcode == "" || fibcode == "l") {
        if (binning == 1 || binning == 0) {
	    if (vb) print ("Large fiber unbinned files")
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flatl12.list"
		else
		    outfile = "flatl12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib="l12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flatl3.list"
		else
		    outfile = "flatl3_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="NNF",apfib="l3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
        if (binning == 2 || binning == 0) {
	    if (vb) print ("Large fiber binned files")
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flatlb12.list"
		else
		    outfile = "flatlb12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib="lb12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flatlb3.list"
		else
		    outfile = "flatlb3_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="NNF",apfib="lb3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
	}

# Medium fiber flat field images
    if (fibcode == "" || fibcode == "m") {
        if (binning == 1 || binning == 0) {
	    if (vb) print ("Medium fiber unbinned files")
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flatm12.list"
		else
		    outfile = "flatm12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib="m12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flatm3.list"
		else
		    outfile = "flatm3_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="NNF",apfib="m3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
        if (binning == 2 || binning == 0) {
	    if (vb) print ("Medium fiber binned files")
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flatmb12.list"
		else
		    outfile = "flatmb12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib=",mb12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flatmb3.list"
		else
		    outfile = "flatmb3_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="NNF",apfib="mb3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
	}

# Small fiber flat field images
    if (fibcode == "" || fibcode == "s") {
        if (binning == 1 || binning == 0) {
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flats12.list"
		else
		    outfile = "flats12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib="s12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flats3.list"
		else
		    outfile = "flats3_" // exp // ".list"
		if (!access ("flats3.list"))
		    trslist (ims,fibkey="NNF",apfib="sb3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
        if (binning == 2 || binning == 0) {
	    if (aps == "1" || aps == "2" || aps == "12" || aps == "123") {
		if (exp == "")
		    outfile = "flatsb12.list"
		else
		    outfile = "flatsb12_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="FFN",apfib="sb12",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    if (aps == "3" || aps == "123") {
		if (exp == "")
		    outfile = "flatsb3.list"
		else
		    outfile = "flatsb3_" // exp // ".list"
		if (!access (outfile) || ow)
		    trslist (ims,fibkey="NNF",apfib="sb3",outlist=outfile,exptime=exp,overwrite=ow,deltemp=dt,verbose=vb,debug=db)
		else if (debug)
		    print ("TFLATLIST: ",outfile," already exists")
		}
	    }
	}

    end

# Jan 11 2008	New task
# Apr 30 2008	Add fibsize and binning parameters to speed processing
# May 23 2008	Add overwrite, debug, apertures, and apfib parameters
# May 23 2008	Change name from tflatfiles to tflatlist (like tharlist)
# May 23 2008	Change fibsize parameter to fibcode
# May 23 2008	Add exptime to make lists for cosmic ray removal
