# tres/Scripts/tharadd.cl
# March 11, 2009
# By Doug Mink

# THARADD -Add pixel shift in a ThAr spectrum to the appropriate table

procedure tharadd (spectrum, reference, shift)

file	spectrum=""	{prompt="ThAr spectrum file to add to shift table"}
file	reference=""	{prompt="ThAr spectrum from which shift is computed"}
real	shift=0.0	{prompt="Shift in pixels from reference spectrum"}
string	outfile=""	{prompt="Returned name of output table file"}
bool    verbose=yes	{prompt="Print process progress (yes or no)"}
bool    debug=no	{prompt="Print debugging info (yes or no)"}

struct *tharfile

begin

    file outtemp, refspec, oldfile
    string ims, afx, s1, s2, s3, s4, fname, rname
    string dline1, dline2, spec, fibsize, bincode, refthar, mjds
    bool vb, ow, db, dt, first, added
    int maxlfile, nfile, maxrfile, iline
    real mjd, mjdi

    vb = verbose
    db = debug
    spec = spectrum
    refspec = reference

    tfib1 (spec)
    afx = tfib1.apfib
    mjd = tfib1.mjdmid
    smatch (spec, "sum") 
    if (smatch.found) {
	if (vb)
	    print ("THARADD: ", spec, " is a sum and not added to table")
	return
	}

# Make file column headings
    fname = "filename"
    maxlfile = 20
    for (i = 9; i <= maxlfile; i = i + 1) {
	fname = fname // " "
	}
    rname = "refthar"
    maxrfile = 30
    for (i = 8; i <= maxrfile; i = i + 1) {
	rname = rname // " "
	}
    dline1 = fname // "	" // rname // " 	mjdmid      	shift    "
    fname = "-"
    for (i = 2; i <= maxlfile; i = i + 1) {
	fname = fname // "-"
	}
    rname = "-"
    for (i = 2; i <= maxrfile; i = i + 1) {
	rname = rname // "-"
	}
    dline2 = fname // "	" // rname // "	------------	---------"

# Make name of pixel shift table for ThAr files with this configuration
    outfile = "comp" // afx // ".shift"
    if (access (outfile)) {
	if (vb)
	    print ("THARADD: Updating ",outfile)
	outtemp = mktemp ("tempcomp") // ".shift"
	iline = 0
	added = no
	tharfile = outfile
	while (fscan (tharfile, s1, s2, s3, s4) != EOF) {
	    iline = iline + 1

# Print two lines of heading 
	    if (iline == 1) {
		print (dline1, >outtemp)
		if (debug)
		    print (dline1)
		}
	    else if (iline == 2) {
		print (dline2, >>outtemp)
		if (debug)
		    print (dline2)
		}

# Update shift if this is an existing entry
	    else if (s1 == spec) {
		round (mjd, "%12.6f")
		mjds = round.value
		round (shift, "%9.6f")
		print (spec,"	", refspec, "	", mjds,"	",round.value, >>outtemp)
		if (db) {
		    print ("THARADD: Replacing line in ", outtemp)
		    print (spec,"	", refspec, "	", mjds,"	",round.value)
		    }
		added = yes
		}

# If earlier time, copy input to output
	    else {
		mjdi = real (s3)
		if (mjdi < mjd) {
		    print (s1,"	", s2, "	", s3, "	", s4, >>outtemp)
		    if (db)
			print (s1,"	", s2, "	", s3, "	", s4)
		    }

# If later time and new entry not added yet, add it
		if (mjdi > mjd && !added) {
		    round (mjdmid, "%12.6f")
		    mjds = round.value
		    round (shift, "%9.6f")
		    if (db) {
			print ("THARADD: Inserting into ", outtemp)
			print (spec,"	", refspec, "	", mjds,"	",round.value)
			}
		    print (spec,"	", refspec, "	", mjds,"	",round.value, >>outtemp)
		    added = yes
		    }

# If later time, copy input to output
		if (mjdi > mjd) {
		    print (s1,"	", s2, "	", s3, "	", s4, >>outtemp)
		    if (db)
			print (s1,"	", s2, "	", s3, "	", s4)
		    }
		}
	    }

# If there was no entry later than the new one, append it
	if (!added) {
	    round (mjd, "%12.6f")
	    mjds = round.value
	    round (shift, "%9.6f")
	    print (spec,"	", refspec, "	", mjds,"	",round.value, >>outtemp)
	    if (db) {
		print ("THARADD: Appending to ", outtemp)
		print (spec,"	", refspec, "	", mjds,"	",round.value)
		}
	    }
	oldfile = outfile // ".old"
	if (access (oldfile))
	    delete (oldfile)
	rename (outfile, oldfile)
	rename (outtemp, outfile)
	}

# If file does not exist, create it, write heading, and add line of data
    else {
	if (vb)
	    print ("THARADD: Creating ",outfile)
	print (dline1, >outfile)
	print (dline2, >>outfile)
	round (mjd, "%12.6f")
	mjds = round.value
	round (shift, "%9.6f")
	print (spec,"	", refspec, "	", mjds,"	",round.value, >>outfile)
	}

end

# Mar 11 2009	New task based on THARTABLE
