# TPROC1 - Run image processing tasks on both TRES amps, then combine them
# May 23, 2008
# By Doug Mink, SAO Telescope Data Center

procedure tproc1 (image)

string	image=""	{prompt="Image to process"}

bool	fixpreamp=no	{prompt="Make preamp correction (yes or no)"}
bool	fixpix=yes	{prompt="Fix bad pixels (yes or no)"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveraw=yes	{prompt="Save unprocessed image files (y or n)"}
bool	saveold=yes	{prompt="Save intermediate image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning
bool	vb, dt, sv, fx, svr, fp

	vb = verbose
	dt = deltemp
	sv = saveold
	svr = saveraw
	imx = ".fits"
	imin = image
	fx = fixpix
	fp = fixpreamp

	if (imin == "") {
	    print ("TPROC1: Run CCDPROC on multi-extension TRES image")
	    print ("Usage: tproc1 image.fits")
	    return
	    }

	if (!defpac ("mscred"))
	    mscred

# Do not process image twice
	imgetp.default = "0"
	imsec = imin // "[0]"
	imgetp (imsec, "TPROC")
	if (imgetp.value != "0") {
	    if (vb) {
		print ("TPROC1: ", imin, " has already been processed")
		print ("TPROC = ", imgetp.value)
		}
	    return
	    }
	else {
	    imgetp (imsec, "TPROC1")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TPROC1: ", imin, " has already been processed")
		    print ("TPROC1 = ", imgetp.value)
		    }
		return
		}
	    }

# Process image file

#  Save original file, if requested, making Raw/ directory, if needed
	if (svr) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    imr = "Raw/" // imin
	    if (!access (imr)) {
		if (vb)
		    print ("TPROC1: Copying ",imin," to Raw/"
		copy (imin, imr)
		}
	    else if (vb)
		print ("TPROC1: ",imin," is in Raw/ already"
	    }

# Fix preamplifier problem which puts flux in overscan region
        if (fixpreamp)
            tpreamp1 (imin,deltemp=dt,saveold=no,verbose=vb)

#  Save original file, if requested, making Raw/ directory, if needed
	if (sv) {
	    if (!access ("Unproc"))
		mkdir ("Unproc")
	    imr = "Unproc/" // imin
	    if (!access (imr)) {
		if (vb)
		    print ("TPROC1: Copying ",imin," to Unproc/"
		copy (imin, imr)
		}
	    else if (vb)
		print ("TPROC1: ",imin," is in Unproc/ already"
	    }

# Find root name of image file
	imroot = imin
	smatch (imroot, imx)
	if (smatch.found) {
	    imroot = substr (imroot,1,smatch.start-1)
	    }
	else {
	    imroot = imin
	    imin = imin // imx
	    }
#	if (vb) {
#	    print ("TPROC1: image root is ",imroot)
#	    }

# Specify FITS extensions for the single-preamplifier images
	amp1 = imin // "[1]"
	amp2 = imin // "[2]"

# Find out binning of image
	if (fixbias) {
	    imgetp.default = "0"
	    imgetp (amp1, "CCDSUM")
	    if (imgetp.value != "0") {
		if (imgetp.value == "1 1")
		    binning = "1"
		else
		    binning = "2"
		}
	    else
		binning = "1"

# Fix bias coordinates in single-preamp images for CCDPROC
	    if (binning == "1") {
		hedit (amp1,"BIASSEC","[1075:1090,3:4608]",update+,verify-,show=vb)
		hedit (amp2,"BIASSEC","[1075:1090,3:4608]",update+,verify-,show=vb)
		}
	    else {
		hedit (amp1,"BIASSEC","[541:545,2:2313]",update+,verify-,show=vb)
		hedit (amp2,"BIASSEC","[541:545,2:2313]",update+,verify-,show=vb)
		}
	    }

#  Trim, subtract overscan, and fix bad pixels using imred.ccdred.ccdproc
	ccdproc.bpmasks = ""
	ccdproc.ccdtype = ""
	ccdproc.noproc = no
	ccdproc.xtalkcor = no
	if (fx) {
	    ccdproc.fixpix = yes
	    ccdproc.fixfile = "tresdata$badpix"
	    }
	else {
	    ccdproc.fixpix = no
	    ccdproc.fixfile = ""
	    }
	ccdproc.overscan = yes
	ccdproc.trim = yes
	ccdproc.zerocor = no
	ccdproc.darkcor = no
	ccdproc.flatcor = no
	ccdproc.sflatcor = no
	ccdproc.split = no
	ccdproc.merge = no
	ccdproc.xtalkfile = ""
	ccdproc.saturation = "INDEF"
	ccdproc.sgrow = 0
	ccdproc.bleed = "INDEF"
	ccdproc.btrail = 20
	ccdproc.bgrow = 0
	ccdproc.biassec = "!BIASSEC"
	ccdproc.trimsec = "!TRIMSEC"
	ccdproc.zero = "ZERO"
	ccdproc.dark = "DARK"
	ccdproc.flat = "FLAT"
	ccdproc.sflat = ""
	ccdproc.minreplace = 1.
	ccdproc.interactive = no
	ccdproc.function = "legendre"
	ccdproc.order = 1
	ccdproc.sample = "*"
	ccdproc.naverage = 1
	ccdproc.niterate = 5
	ccdproc.low_reject = 3.
	ccdproc.high_reject = 3.
	ccdproc.grow = 0.
	ccdproc.fd = ""
	ccdproc.fd2 = ""

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
	if (vb)
	    print ("TPROC1: Running ccdproc for fixpix, overscan, and trim")
	temp2 = imroot // "_out.fits"
	ccdproc (imin, output=temp2)
	if (!access (temp2)) {
	    if (vb)
		print ("TPROC1: CCDPROC failed; trying again")
	    ccdproc (imin, output=temp2)
	    }
	if (!access (temp2)) {
	    if (vb)
		print ("TPROC1: CCDPROC failed; trying again")
	    ccdproc (imin, output=temp2)
	    }
	if (!access (temp2)) {
	    if (vb)
		print ("TPROC1: CCDPROC failed; trying again")
	    ccdproc (imin, output=temp2)
	    }
	if (vb)
	    print ("TPROC1: fixpix, overscan, and trim ", imin," -> ", temp2)
	imdel (imin)
	rename (temp2, imin)

# Set history line including time
	gettime (0.0, timediff=99)
	history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"

# Add history line including time
	hedit (imsec, "TPROC", history, add+, verify-)
end

# Feb 25 2008	Extract processing code from TPREPROC
# Apr 30 2008	Make Raw/ directory only if saving a file there
# Apr 30 2008	Move preamp fix from TPROC
# May 16 2008	Add input time argument to gettime() 
# May 23 2008	Fix TPROC write to header keyword
# Dec 15 2008	Add timediff argument to GETTIME to get current time
