# File t_tcosfix.x
# August 7, 2008
# By Doug Mink after Nelson Caldwell

include <error.h>
include <ctype.h>
include <imhdr.h>
include <imio.h>
include <mach.h>

define	MAXROWL		4096
define	MAXLINE		4700
define	MAXBUFF		100000
define	SZ_HSTRING	68

# TCOSFIX -- Locate cosmic rays in a pair of images by looking at
# subsections for which IRAF has already determined the sigma
# Output new file(s) where pixels where the image has a value
# over a second image plus the detection limit set by tcoslim
# are replaced by either the scaled second image or scaled median value

procedure t_tcosfix ()

pointer	im1, im2, im3, im4, pdata
real	data1[MAXROWL,MAXLINE],data2[MAXROWL,MAXLINE]
real	datamed[MAXROWL,MAXLINE], datalim[MAXROWL,MAXLINE]
real	datat1[MAXBUFF],datat2[MAXBUFF], datatm[MAXBUFF]
char	imfile1[SZ_FNAME]
char	imfile2[SZ_FNAME]
char	medfile[SZ_FNAME]
char	limfile[SZ_FNAME]
char	hstr0[SZ_HSTRING], hstr[SZ_HSTRING], dstr[SZ_HSTRING]
long	v[IM_MAXDIM]
bool	verbose, debug, proc2, medrep
int	nc, nl, ix, ix2, iy, iy2, i, j, ij, idx, idy
int	nbad1, nbad2, ijmax, ii, ii1, ii2, jj, jj1, jj2
real	ratio, ratm1, ratm2
real	dmax, stat1, stat2, statm, diff, dthresh
double	xsum1, xsum2, xsumm
pointer	outline
pointer	imgl2r(), immap()
int	clgeti(), impnlr()
bool	clgetb()
long	clktime()
include <szdtype.inc>

begin

# Get parameters
	call clgstr ("image1", imfile1, SZ_FNAME)
	call clgstr ("image2", imfile2, SZ_FNAME)
	call clgstr ("medfile", medfile, SZ_FNAME)
	call clgstr ("limfile", limfile, SZ_FNAME)
	proc2 = clgetb ("proc2")
	medrep = clgetb ("medrep")
	idx = clgeti ("xbox")
	idy = clgeti ("ybox")
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")
	dthresh = 10000.0

# Open the first image
	iferr (im1 = immap (imfile1, READ_WRITE, 0)) {
	    call printf ("TCOSFIX: Cannot read %s\n")
		call pargstr (imfile1)
	    call clputi ("error", 1)
	    return
	    }
	nc = IM_LEN(im1,1)
	nl = IM_LEN(im1,2)
	nbad1 = 0
	if (debug) {
	    call printf ("TCOSFIX: %d x %d x %d input image %s opened\n")
		call pargi (nc)
		call pargi (nl)
		call pargi (ty_size[IM_PIXTYPE(im1)])
		call pargstr (imfile1)
	    call flush (STDOUT)
	    }

# Read the first input image
	do j = 1, nl {
	    pdata = imgl2r (im1, j)
	    do i = 1, nc {
		data1[i,j] = Memr[pdata + i - 1]
		}
	    }
	if (verbose) {
	    call printf ("TCOSFIX: Input image %s read\n")
		call pargstr (imfile1)
	    call flush (STDOUT)
	    } 

# Open the second input image
	iferr (im2 = immap (imfile2, READ_WRITE, 0)) {
	    call printf ("TCOSFIX: Cannot read %s\n")
		call pargstr (imfile2)
	    call clputi ("error", 2)
	    return
	    }
	nc = IM_LEN(im2,1)
	nl = IM_LEN(im2,2)
	nbad2 = 0
	if (debug) {
	    call printf ("TCOSFIX: %d x %d x %d input image %s opened\n")
		call pargi (nl)
		call pargi (nc)
		call pargi (ty_size[IM_PIXTYPE(im2)])
		call pargstr (imfile2)
	    call flush (STDOUT)
	    }

# Read the second input image
	do j = 1, nl {
	    pdata = imgl2r (im2, j)
	    do i = 1, nc {
		data2[i,j] = Memr[pdata + i - 1]
		}
	    }
	if (verbose) {
	    call printf ("TCOSFIX: Input image %s read\n")
		call pargstr (imfile2)
	    call flush (STDOUT)
	    }

# Open the limit image
	iferr (im4 = immap (limfile, READ_ONLY, 0)) {
	    call printf ("TCOSFIX: Cannot read %s\n")
		call pargstr (limfile)
	    call clputi ("error", 4)
	    return
	    }
	nc = IM_LEN(im4,1)
	nl = IM_LEN(im4,2)
	if (debug) {
	    call printf ("TCOSFIX: %d x %d x %d limit image %s opened\n")
		call pargi (nl)
		call pargi (nc)
		call pargi (ty_size[IM_PIXTYPE(im4)])
		call pargstr (limfile)
	    call flush (STDOUT)
	    }

# Read the limit image
	do j = 1, nl {
	    pdata = imgl2r (im4, j)
	    do i = 1, nc {
		datalim[i,j] = Memr[pdata + i - 1]
		}
	    }
	if (verbose) {
	    call printf ("TCOSFIX: Limit image %s read\n")
		call pargstr (limfile)
	    call flush (STDOUT)
	    }

# Close the limit image
	call imunmap (im4)

# Open the median image if it is used
	if (medrep) {
	    iferr (im3 = immap (medfile, READ_ONLY, 0)) {
		call printf ("TCOSFIX: Cannot read %s\n")
		    call pargstr (medfile)
		call clputi ("error", 3)
		return
		}
	    nc = IM_LEN(im3,1)
	    nl = IM_LEN(im3,2)

	    if (debug) {
		call printf ("TCOSFIX: %d x %d x %d median image %s opened\n")
		    call pargi (nl)
		    call pargi (nc)
		    call pargi (ty_size[IM_PIXTYPE(im3)])
		    call pargstr (medfile)
		call flush (STDOUT)
		}

# Read the median image
	    do j = 1, nl {
		pdata = imgl2r (im3, j)
		do i = 1, nc {
		    datamed[i,j] = Memr[pdata + i - 1]
		    }
		}

# Close the median image
	    call imunmap (im3)

	    if (verbose) {
		call printf ("TCOSFIX: Median image %s read\n")
		    call pargstr (medfile)
		call flush (STDOUT)
		}
	    }

# Compare sections dx pixels wide by dy pixels long
 	for (ix = 1; ix <= nc; ix = ix + idx) {
	    ix2 = ix + idx - 1
	    if (ix2 > nc)
		ix2 = nc

 	    for (iy = 1; iy <= nl; iy = iy + idy) {
		iy2 = iy + idy - 1
		if (iy2 > nl)
		    iy2 = nl

# Form scaling ratio from a box
		ij=0
		do i = ix, ix2 {
		    do j = iy , iy2 {
			ij = ij + 1
			datat2[ij] = data2[i,j]
			datat1[ij] = data1[i,j]
			datatm[ij] = datamed[i,j]
			}
		    }

# Form a sort of median in both boxes to get the mean ratio of fluxes
		call asrtr (datat1, datat1, ij)
		call asrtr (datat2, datat2, ij)

# Drop the highest fluxes in case they are cosmic rays
		ijmax = 0.6 * ij
		xsum1 = 0.d0
		xsum2 = 0.d0
		do ij = 1, ijmax {
		    xsum1 = xsum1 + double (datat1[ij])
		    xsum2 = xsum2 + double (datat2[ij])
		    }
		stat1 = real (xsum1 / double (ijmax))
		stat2 = real (xsum2 / double (ijmax))

# Ratio of fluxes between images
		if ((stat2 > 20.0) || (stat1 > 20.0))
		    ratio = stat1 / stat2
		else
		    ratio = 1.0

#		if (debug) {
#		    call printf ("x= %d y=%d ratio = %.4f = %.2f / %.2f\n")
#			call pargi (ix)
#			call pargi (iy)
#			call pargr (ratio)
#			call pargr (stat1)
#			call pargr (stat2)
#		    call flush (STDOUT)
#		    }

# Compute median of median image and ratios of fluxes to median image
# only if using median
		if (medrep) {
		    call asrtr (datatm, datatm, ij)
		    xsumm = 0.d0
		    do ij = 1, ijmax {
			xsumm = xsumm + double (datatm[ij])
			}
		    statm = real (xsumm / double (ijmax))
		    if ((statm > 20.0) || (stat1 > 20.0))
			ratm1 = stat1 / statm
		    else
			ratm1 = 1.0
		    if ((statm > 20.0) || (stat2 > 20.0))
			ratm2 = stat2 / statm
		    else
			ratm2 = 1.0
		    }

# Now look at pixels in that box
		do i = ix, ix2 {
		    do j = iy, iy2 {

# The statistical check for a cosmic is
#  pix1 - pix2*ratio > n * 1.4 * sqrt(median) 
# where ratio is the flux ratio of image 1 to image 2 (or the median image)
# in a dx x dy box around the pixel, and the right side is precomputed by
# the tres.tcoslim task
			dmax = datalim[i,j]

# If medrep=yes, compare pixel value to scaled value of median image pixel
			if (medrep) {
			    diff = data1[i,j] - (datamed[i,j] * ratm1)
			    if (diff > dmax) {
				if (debug && diff > dthresh) {
				    call printf ("badpix (%4d,%4d): %.2f -> %.2f\n")
					call pargi (i)
					call pargi (j)
					call pargr (data1[i,j])
					call pargr (ratm1 * datamed[i,j])
				    call flush (STDOUT)
				    }
				if (i == 1)
				    ii1 = 1
				else
				    ii1 = i - 1
				if (i == nc)
				    ii2 = nc
				else
				    ii2 = i + 1
				if (j == 1)
				    jj1 = 1
				else
				    jj1 = j - 1
				if (j == nl)
				    jj2 = nl
				else
				    jj2 = j + 1
				do jj = jj1, jj2, 1 {
				    do ii = ii1, ii2, 1 {
					data2[ii,jj] = ratm1 * datamed[ii,jj]
					}
				    }
				nbad1 = nbad1 + 1
				}
			    }

# Otherwise, compare pixel value to scaled value of pixel from second image
			else {
			    diff = data1[i,j] - (data2[i,j] * ratio)
			    if (diff > dmax) {
				if (debug && diff > dthresh) {
				    call printf ("Badpix1 (%4d,%4d): %.2f -> %.2f\n")
					call pargi (i)
					call pargi (j)
					call pargr (data1[i,j])
					call pargr (ratio * data2[i,j])
				    call flush (STDOUT)
				    }
				if (i == 1)
				    ii1 = 1
				else
				    ii1 = i - 1
				if (i == nc)
				    ii2 = nc
				else
				    ii2 = i + 1
				if (j == 1)
				    jj1 = 1
				else
				    jj1 = j - 1
				if (j == nl)
				    jj2 = nl
				else
				    jj2 = j + 1
				do jj = jj1, jj2, 1 {
				    do ii = ii1, ii2, 1 {
					data1[ii,jj] = ratio * data2[ii,jj]
					}
				}
				nbad1 = nbad1 + 1
				}
			    }

# If doing two files at once
			if (proc2) {

# If medrep=yes, compare pixel value to scaled value of median image pixel
			    if (medrep) {
				diff = data2[i,j] - (datamed[i,j] * ratm2)
				if (diff > dmax) {
				    if (debug && diff > dthresh) {
					call printf ("Badpix2 (%4d,%4d): %.2f -> %.2f\n")
					    call pargi (i)
					    call pargi (j)
					    call pargr (data2[i,j])
					    call pargr (ratm2 * datamed[i,j])
					call flush (STDOUT)
					}
				    if (i == 1)
					ii1 = 1
				    else
					ii1 = i - 1
				    if (i == nc)
					ii2 = nc
				    else
					ii2 = i + 1
				    if (j == 1)
					jj1 = 1
				    else
					jj1 = j - 1
				    if (j == nl)
					jj2 = nl
				    else
					jj2 = j + 1
				    do jj = jj1, jj2, 1 {
					do ii = ii1, ii2, 1 {
					    data2[ii,jj] = ratm2 * datamed[ii,jj]
					    }
					}
				    nbad2 = nbad2 + 1
				    }
				}

# Otherwise, compare pixel value to scaled value of pixel from first image
			    else {
				diff = data2[i,j] - (data1[i,j] / ratio)
				if (diff > dmax) {
				    if (debug && diff > dthresh) {
					call printf ("Badpix2 (%4d,%4d): %.2f -> %.2f\n")
					    call pargi (i)
					    call pargi (j)
					    call pargr (data2[i,j])
					    call pargr (data1[i,j] / ratio)
					call flush (STDOUT)
					}
				    if (i == 1)
					ii1 = 1
				    else
					ii1 = i - 1
				    if (i == nc)
					ii2 = nc
				    else
					ii2 = i + 1
				    if (j == 1)
					jj1 = 1
				    else
					jj1 = j - 1
				    if (j == nl)
					jj2 = nl
				    else
					jj2 = j + 1
				    do jj = jj1, jj2, 1 {
					do ii = ii1, ii2, 1 {
					    data2[ii,jj] = data1[ii,jj] / ratio
					    }
					}
				    nbad2 = nbad2 + 1
				    }
				}
			    }
			}
		    }
		}
	    }

	if (verbose) {
	    call printf ("TCOSFIX: Found and changed %d bad pixels in %s\n")
		call pargi (nbad1)
		call pargstr (imfile1)
	    call flush (STDOUT)
	    if (proc2) {
		call printf ("TCOSFIX: Found and changed %d bad pixels in %s\n")
		    call pargi (nbad2)
		    call pargstr (imfile2)
		call flush (STDOUT)
		}
	    }

# Update the first image file one line at a time
	if (nbad1 > 0) {
	    call amovkl (long(1), v, IM_MAXDIM)
	    do j = 1, nl {

# Get buffer for this line of image
		v[2] = j
		if (impnlr (im1, outline, v) == EOF) {
		    call printf ("TCOSFIX: Error writing line %d of %s\n")
			call pargi (j)
			call pargstr (imfile1)
		    call flush (STDOUT)
		    call clputi ("error", 6)
		    return
		    }
		call achtrr (data1[1,j], Memr[outline], nc)
		}
	    IM_CTIME(im1) = clktime (long(0))
	    if (verbose) {
		call printf ("TCOSFIX: Updated %d x %d image %s\n")
		    call pargi (IM_LEN(im1,1))
		    call pargi (IM_LEN(im1,2))
		    call pargstr (imfile1)
		    call flush (STDOUT)
		}
	    IM_UPDATE(im1) = YES
	    call logtime (dstr, SZ_HSTRING)
	    if (medrep)
		call sprintf (hstr,SZ_HSTRING,"%s %d cosmic rays replaced from median")
	    else
		call sprintf (hstr,SZ_HSTRING,"%s %d cosmic rays replaced from file")
		call pargstr (dstr)
		call pargi (nbad1)
	    hstr0[1] = EOS
	    call imgspar (im1,"TCOSMIC",hstr0,SZ_HSTRING)
	    if (hstr0[1] != EOS)
		call imputh (im1, "HISTORY", hstr0)
	    call imastr (im1, "TCOSMIC", hstr)
	    call imaddi (im1, "TCOSREP", nbad1)
	    if (medrep)
		call imastr (im1, "TCOSFILE", medfile)
	    else
		call imastr (im1, "TCOSFILE", imfile2)
	    call imunmap (im1)
	    }

# Update the second image file one line at a time
	if (proc2 && nbad2 > 0) {
	    call amovkl (long(1), v, IM_MAXDIM)
	    do j = 1, nl {
		v[2] = j
		if (impnlr (im2, outline, v) == EOF) {
		    call printf ("TCOSFIX: Error writing line %d of %s\n")
			call pargi (j)
			call pargstr (imfile2)
		    call flush (STDOUT)
		    call clputi ("error", 7)
		    return
		    }
		call achtrr (data2[1,j], Memr[outline], nc)
		}
	    IM_CTIME(im2) = clktime (long(0))
	    if (verbose) {
		call printf ("TCOSFIX: Updated %d x %d image %s\n")
		    call pargi (IM_LEN(im2,1))
		    call pargi (IM_LEN(im2,2))
		    call pargstr (imfile2)
		call flush (STDOUT)
		}
	    IM_UPDATE(im2) = YES
	    call logtime (dstr, SZ_HSTRING)
	    if (medrep)
		call sprintf (hstr,SZ_HSTRING,"%s %d cosmic rays replaced from median")
	    else
		call sprintf (hstr,SZ_HSTRING,"%s %d cosmic rays replaced from file")
		call pargstr (dstr)
		call pargi (nbad2)
	    hstr0[1] = EOS
	    call imgspar (im2,"TCOSMIC",hstr0,SZ_HSTRING)
	    if (hstr0[1] != EOS)
		call imputh (im2, "HISTORY", hstr0)
	    call imastr (im2, "TCOSMIC", hstr)
	    if (medrep)
		call imastr (im2, "TCOSFILE", medfile)
	    else
		call imastr (im2, "TCOSFILE", imfile1)
	    call imunmap (im2)
	    }

end
# Nov  8 2007	New program after tres.tcosbad
# Nov 13 2007	Fix 8 adjacent pixels to each bad one
# Nov 14 2007	Write operation description to image header(s)
# Nov 15 2007	Check row and column limits when replacing pixels
# Nov 15 2007	Print pixel size instead of code

# Jul 10 2008	Add TCOSREP with number of bad pixels replaced
# Aug  7 2008	Drop declaration of unused function clgetr()
