# File tres/ecidentify/t_ecrefit.x
# September 18, 2008
# By Doug Mink after IRAF T_ECREIDENTIFY

include	<error.h>
include <smw.h>
include	"ecidentify.h"

# T_ECREFIT -- Refit echelle features from shifted database entry

procedure t_ecrefit ()

int	images			# List of images

int	i, fd, nfeatures1, nfeatures2
pointer	ref			# Reference image
pointer	sp, log, ec
int	verbose, interactive, fixedorder

int	imtopenp(), ec_getim(), clpopnu(), clgfil(), open()
int	strmatch(), btoi()
double	ec_rms()
bool	clgetb()
errchk	ec_dbread(), ec_gdata(), ec_fitdata()

begin
	call smark (sp)
	call salloc (ref, SZ_FNAME, TY_CHAR)
	call salloc (log, SZ_FNAME, TY_CHAR)

	# Allocate the basic data structure.
	call ec_init (ec)

	# Initialize fitting
	call ecf_seti ("niterate", 0)
	call ecf_setd ("low", 3.D0)
	call ecf_setd ("high", 3.D0)

	# Get task parameters.
	images = imtopenp ("images")
	call clgstr ("database", Memc[EC_DATABASE(ec)], SZ_FNAME)
	EC_LOGFILES(ec) = clpopnu ("logfiles")
	interactive = YES
	fixedorder = NO
	verbose = btoi (clgetb ("verbose"))

	# Write logfile header.
	while (clgfil (EC_LOGFILES(ec), Memc[log], SZ_FNAME) != EOF) {
	    iferr (fd = open (Memc[log], APPEND, TEXT_FILE)) {
		call erract (EA_WARN)
		next
	    }
	    call sysid (Memc[log], SZ_LINE)
	    call fprintf (fd, "\nECREFIT: %s\n")
		call pargstr (Memc[log])
	    call fprintf (fd,
		"  Reference image = self, Refit = yes\n")
		call pargb (EC_REFIT(ec) == YES)
	    call fprintf (fd, "%20s  %7s %7s  %7s\n")
		call pargstr ("Image")
		call pargstr ("Found")
		call pargstr ("Fit")
		call pargstr ("RMS")
	    call close (fd)
	}

	# Refit features in each spectrum.
	while (ec_getim (images, Memc[EC_IMAGE(ec)], SZ_FNAME) != EOF) {
	    call strcpy (Memc[EC_IMAGE(ec)], Memc[ref])
	    i = strmatch (Memc[ref], ".fits")
	    if (i > 0)
		Memc[ref+i-1] = EOS
	    call ec_gdata (ec)

	    # Read line assignments and existing fit
	    call ec_dbread (ec, Memc[ref], verbose)

	    # Re-fit lines to shifted positions
	    call ec_dofit (ec, interactive, fixedorder)

	    # Write the re-fit database entry for the image.
#	    call delete (Memc[EC_IMAGE(ec)])
	    call ec_dbwrite (ec, Memc[EC_IMAGE(ec)], verbose)

	    # Record log information if a log file descriptor is given.
	    call clprew (EC_LOGFILES(ec))
	    while (clgfil (EC_LOGFILES(ec), Memc[log], SZ_FNAME) != EOF) {
	        iferr (fd = open (Memc[log], APPEND, TEXT_FILE)) {
		    call erract (EA_WARN)
		    next
	        }
	        call fprintf (fd,
		    "%20s  %3d/%-3d %3d/%-3d %7.3g\n")
		    call pargstr (Memc[EC_IMAGE(ec)])
		    call pargi (EC_NFEATURES(ec))
		    call pargi (nfeatures1)
		    call pargi (nfeatures2)
		    call pargi (EC_NFEATURES(ec))
		    call pargd (ec_rms(ec))
	        call close (fd)
	    }

	    call smw_close (MW(EC_SH(ec)))
	    do i = 1, EC_NLINES(ec)
		MW(SH(ec,i)) = NULL
	}

	call dgsfree (EC_ECF(ec))
	call clpcls (EC_LOGFILES(ec))
	call ec_free (ec)
	call imtclose (images)
	call sfree (sp)
end
# Aug 15 2008	New task
# Sep 11 2008	Fix fitting and add verbose parameter
# Sep 18 2008	Drop unused declarations
